"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileBSON = void 0;
const json_schema_to_typescript_1 = require("json-schema-to-typescript");
/**
 * JSON Schema types (type keyword)
 * https://json-schema.org/understanding-json-schema/reference/type.html
 *
 * string
 * number
 * object
 * array
 * boolean
 * null
 *
 * BSON types (bsonType keyword)
 * https://docs.mongodb.com/manual/reference/operator/query/jsonSchema/#available-keywords
 * https://docs.mongodb.com/manual/reference/operator/query/type/#document-type-available-types
 *
 * [X] number
 * [ ] double
 * [X] string
 * [ ] object
 * [ ] array
 * [ ] binData
 * [ ] undefined  -- Deprecated
 * [ ] objectId
 * [X] bool
 * [X] date
 * [X] null
 * [ ] regex
 * [ ] dbPointer  -- Deprecated
 * [ ] javascript
 * [ ] symbol     -- Deprecated
 * [ ] javascriptWithScope
 * [ ] int
 * [ ] timestamp
 * [ ] long
 * [X] decimal
 * [ ] minKey
 * [ ] maxKey
 *
 * BSON types -> TS
 */
const bsonToTs = new Map([
    ["number", "number"],
    // ["double", "Double"],
    ["string", "string"],
    ["bool", "boolean"],
    ["date", "Date"],
    ["null", "null"],
    ["decimal", "Decimal128"],
]);
/**
 * Generate the 'tsType' field for bson types.
 * https://github.com/bcherny/json-schema-to-typescript#custom-schema-properties
 */
function buildTsType(bsonType) {
    // Handle array of BSON types
    //
    // This is necessary since
    // https://github.com/bcherny/json-schema-to-typescript does not support
    // 'tsType' as an array
    if (Array.isArray(bsonType)) {
        return bsonType.map((e) => buildTsType(e)).join("|");
    }
    if (typeof bsonType !== "string")
        return null;
    // Return the 'tsType' based on the BSON type
    return bsonToTs.get(bsonType) || null;
}
/**
 * Annotates the Schema objects with 'tsType' properties based on the 'bsonType'
 */
function addTsType(schema) {
    if (Array.isArray(schema)) {
        return schema.map(addTsType);
    }
    if (typeof schema === "object" && schema !== null) {
        const bsonType = "bsonType" in schema ? schema.bsonType : undefined;
        const isEnum = Reflect.has(schema, "enum");
        const tsType = buildTsType(bsonType);
        // Add 'tsType' fields to objects, except enums
        const v = !isEnum && tsType ? Object.assign(Object.assign({}, schema), { tsType }) : schema;
        return Object.entries(v).reduce((acc, [key, value]) => {
            return Object.assign(Object.assign({}, acc), { [key]: addTsType(value) });
        }, {});
    }
    return schema;
}
function hasDecimal128(schema) {
    if (schema === null || typeof schema !== "object") {
        return false;
    }
    if (Array.isArray(schema)) {
        return schema.some(hasDecimal128);
    }
    return Object.entries(schema).some(([key, value]) => {
        if (key === "bsonType") {
            if (value === "decimal")
                return true;
            if (Array.isArray(value))
                return value.includes("decimal");
        }
        if (typeof value === "object")
            return hasDecimal128(value);
        return false;
    });
}
const bannerCommentLines = [
    "/* eslint-disable */",
    "/* tslint:disable */",
    "/**",
    "* This file was automatically generated by bson-schema-to-typescript.",
    "* DO NOT MODIFY IT BY HAND. Instead, modify the source JSONSchema file,",
    "* and run bson-schema-to-typescript to regenerate this file.",
    "*/",
];
function compileBSON(schema, options) {
    return __awaiter(this, void 0, void 0, function* () {
        // Add 'tsType' fields as needed
        const newSchema = addTsType(schema);
        // Import Decimal128 if the type annotations use it
        const bannerComment = (hasDecimal128(schema)
            ? [...bannerCommentLines, "import { Decimal128 } from 'bson';"]
            : bannerCommentLines).join("\n");
        const opts = Object.assign(Object.assign({}, options), { bannerComment, style: (options === null || options === void 0 ? void 0 : options.prettier) ? options.prettier : null });
        // Generate types
        const output = yield json_schema_to_typescript_1.compile(newSchema, "", opts);
        return output;
    });
}
exports.compileBSON = compileBSON;
//# sourceMappingURL=compile.js.map